/*
 *  for MB9BF618T on FRK-FM3
 * 
 *   Copyright (C) by Y.Ishioka.
 */
#include "mb9b610t.h"
#include "fm3.h"

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
extern "C" {
#endif
/**INDENT-ON**/
/// @endcond

#define  DEF_ulPin_LED0     13
#define  DEF_ulPin_BUTTON   20
#define  DEF_ulPin_MAX      20

int  fm3_pinSetting[ DEF_ulPin_MAX+1 ] ;

void  fm3_pinMode( unsigned int ulPin, unsigned int ulMode );
void  fm3_digitalWrite( unsigned int ulPin, unsigned int ulVal );
int   fm3_digitalRead( unsigned int ulPin );

/***********************************************************/
/*
 * IO port initialize
 */
void  switch_init( unsigned int bit )
{
    switch( bit ) {
    case 0:
        /* PF6 */
        FM3_GPIO->ADE = FM3_GPIO->ADE & ~0x00000000 ;
        FM3_GPIO->PFRF  &= ~0x0040 ;  /* set GPIO */
        FM3_GPIO->PCRF  |=  0x0040 ;  /* on pull-up */
        FM3_GPIO->DDRF  &= ~0x0040 ;  /* set input mode */
        break ;

    default:
        break ;
    }
}


/*
 * bit : 0 : LOW
 *       1 : HIGH
 */
int  switch_read( unsigned int bit )
{
    int  stat = 0 ;

    switch( bit ) {
    case 0:
        /* PF6 */
        if( (FM3_GPIO->PDORF & 0x0040) != 0 ) {
            stat = 1 ;
        }
        break ;

    default:
        break ;
    }

    return stat ;
}


/***********************************************************/
/*
 * digital IO port initialize
 */
void  fm3_pinMode( unsigned int ulPin, unsigned int ulMode )
{
    unsigned int  val ;

    if( DEF_ulPin_MAX < ulPin ) {
        return ;
    }

    if( ulMode == 0xffff ) {
        return ;
    }

    fm3_pinSetting[ ulPin ] = ulMode + 1 ;

    /* DEF_ulPin_BUTTON && INPUT */
    if( ulPin == DEF_ulPin_BUTTON && ulMode == 0 ) {
        switch_init( 0 ) ;
        return ;
    }

    switch( ulPin ) {
    case 0:  /* P21 */
        val = 0x0002 ;
        FM3_GPIO->PFR2  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR2  &= ~val ; /* set input mode */
            FM3_GPIO->PCR2  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR2  |=  val ; /* set output mode */
            FM3_GPIO->PDOR2 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR2  &= ~val ; /* set input mode */
            FM3_GPIO->PCR2  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 1:  /* P22 */
        val = 0x0004 ;
        FM3_GPIO->PFR2  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR2  &= ~val ; /* set input mode */
            FM3_GPIO->PCR2  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR2  |=  val ; /* set output mode */
            FM3_GPIO->PDOR2 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR2  &= ~val ; /* set input mode */
            FM3_GPIO->PCR2  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 2:  /* P12 */
        val = 0x0004 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 3:
        val = 0x0008 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 4:
        val = 0x0010 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 5:
        val = 0x0020 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 6:
        val = 0x0040 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 7:
        val = 0x0080 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 8:
        val = 0x0100 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 9:
        val = 0x0200 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 10:  /* P1A */
        val = 0x0400 ;
        FM3_GPIO->ADE   &= ~val ; /* off ADC */
        FM3_GPIO->PFR1  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR1  |=  val ; /* set output mode */
            FM3_GPIO->PDOR1 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR1  &= ~val ; /* set input mode */
            FM3_GPIO->PCR1  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 11:  /* P50 */
        val = 0x0001 ;
        FM3_GPIO->PFR5  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR5  |=  val ; /* set output mode */
            FM3_GPIO->PDOR5 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 12:  /* P51 */
        val = 0x0002 ;
        FM3_GPIO->PFR5  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR5  |=  val ; /* set output mode */
            FM3_GPIO->PDOR5 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 13:  /* P52 */
        val = 0x0004 ;
        FM3_GPIO->PFR5  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDR5  |=  val ; /* set output mode */
            FM3_GPIO->PDOR5 &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDR5  &= ~val ; /* set input mode */
            FM3_GPIO->PCR5  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 14:  /* PB0 */
        val = 0x0001 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 15:
        val = 0x0002 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 16:
        val = 0x0004 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 17:
        val = 0x0008 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 18:
        val = 0x0010 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;

    case 19:
        val = 0x0020 ;
        FM3_GPIO->ADE   &= ~(val << 16) ; /* off ADC */
        FM3_GPIO->PFRB  &= ~val ; /* set GPIO  */
        switch( ulMode ) {
        case 0: /* INPUT */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  &= ~val ; /* off pull-up */
            break ;
        case 1: /* OUTPUT */
            FM3_GPIO->DDRB  |=  val ; /* set output mode */
            FM3_GPIO->PDORB &= ~val ; /* clear data bit */
            break ;
        case 2: /* INPUT_PULLUP */
            FM3_GPIO->DDRB  &= ~val ; /* set input mode */
            FM3_GPIO->PCRB  |=  val ; /* set pull-up */
            break ;
        }
        break ;
    }
}


void  fm3_digitalWrite( unsigned int ulPin, unsigned int ulVal )
{
    unsigned int  val = 0 ;

    if( DEF_ulPin_MAX < ulPin ) {
        return ;
    }

    /* check after pinMode() */
    if( fm3_pinSetting[ ulPin ] != 2 ) {
        return ;
    }

    if( ulVal != 0 ) {
        val = 1 ;
    }

    if( ulPin == 13 ) {
        led_set( 0, val ) ;
        return ;
    }

    switch( ulPin ) {
    case 0:  /* P21 */
        FM3_GPIO->PDOR2 = (FM3_GPIO->PDOR2 & ~(1<<(ulPin+1))) ;
        FM3_GPIO->PDOR2 |=  (val<<(ulPin+1)) ;
        break ;

    case 1:  /* P22 */
        FM3_GPIO->PDOR2 = (FM3_GPIO->PDOR2 & ~(1<<(ulPin+1))) ;
        FM3_GPIO->PDOR2 |=  (val<<(ulPin+1)) ;
        FM3_GPIO->ADE &= ~0x80000000 ;  /* disable AN31 */
        break ;

    case 2:  /* P12 */
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    case 10:
        FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~(1<<ulPin)) ;
        FM3_GPIO->PDOR1 |= (val<<ulPin) ;
        break ;

    case 11:  /* P50 */
    case 12:  /* P51 */
    case 13:  /* P52 */
        FM3_GPIO->PDOR5 = (FM3_GPIO->PDOR5 & ~(1<<(ulPin-11))) ;
        FM3_GPIO->PDOR5 |= (val<<(ulPin-11)) ;
        break ;

    case 14:  /* PB0 */
    case 15:
    case 16:
    case 17:
    case 18:
    case 19:
        FM3_GPIO->PDORB = (FM3_GPIO->PDORB & ~(1<<(ulPin-14))) ;
        FM3_GPIO->PDORB |= (val<<(ulPin-14)) ;
        FM3_GPIO->ADE &= ~(1<<(ulPin+2)) ;  /* disable AN1x */
        break ;
    }
}


int  fm3_digitalRead( unsigned int ulPin )
{
    int  val = 0 ;

    if( DEF_ulPin_MAX < ulPin ) {
        return  val ;
    }

    /* check after pinMode() */
    if( !(fm3_pinSetting[ ulPin ] == 1 || fm3_pinSetting[ ulPin ] == 3) ) {
        return  val ;
    }

    if( ulPin == 16 ) {
        if( switch_read( 0 ) ) {
            val = 1 ;  /* high:1 */
        } else {
            val = 0 ;  /* low :0 */
        }
        return  val ;
    }

    switch( ulPin ) {
    case 0:  /* P21 */
    case 1:  /* P22 */
        val = (FM3_GPIO->PDOR2 >> (ulPin+1)) & 0x01 ;
        break ;

    case 2:  /* P12 */
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
    case 10:
        val = (FM3_GPIO->PDOR1 >> ulPin) & 0x01 ;
        break ;

    case 11:  /* P50 */
    case 12:  /* P51 */
    case 13:  /* P52 */
        val = (FM3_GPIO->PDOR5 >> (ulPin-11)) & 0x01 ;
        break ;

    case 14:  /* PB0 */
    case 15:
    case 16:
    case 17:
    case 18:
    case 19:
        val = (FM3_GPIO->PDORB >> (ulPin-14)) & 0x01 ;
        break ;
    }

    return  val ;  /* low :0 */

}


//@}

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
}
#endif
/**INDENT-ON**/
/// @endcond
