/*
 *  for MB9BF618T on FRK-FM3
 * 
 *   Copyright (C) by Y.Ishioka.
 */
#include "mb9b610t.h"
#include "fm3.h"

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
extern "C" {
#endif
/**INDENT-ON**/
/// @endcond


/***********************************************************/
/*
 * data: bit7-4
 * rs  : RS signal
 */
void  LCD_put4bit( unsigned int data, unsigned int rs )
{
    if( rs != 0 ) {
        /* set RS and reset E */
        FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~0x000c) | 0x0004 ;
    } else {
        /* reset RS and reset E */
        FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x000c ;
    }
    WaitUsec( 20 ) ;

    /* set data upper */
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 | 0x0008 ; /* set E */
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~0x00f0) | (data&0x00f0) ;
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x0008 ; /* clear E */
    WaitUsec( 20 ) ;

    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x000c ; /* clear RS,E */
}


/***********************************************************/
/*
 * data: bit7-0
 * rs  : RS signal
 */
void  lcd_putchar( unsigned int data, unsigned int rs )
{
    if( rs != 0 ) {
        /* set RS and reset E */
        FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~0x000c) | 0x0004 ;
    } else {
        /* reset RS and reset E */
        FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x000c ;
    }
    WaitUsec( 20 ) ;

    /* set data upper */
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 | 0x0008 ; /* set E */
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~0x00f0) | (data&0x00f0) ;
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x0008 ; /* clear E */
    WaitUsec( 20 ) ;

    /* set data lower */
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 | 0x0008 ; /* set E */
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = (FM3_GPIO->PDOR1 & ~0x00f0) | ((data<<4)&0x00f0) ;
    WaitUsec( 20 ) ;
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x0008 ; /* clear E */
    WaitUsec( 20 ) ;

    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x000c ; /* clear RS,E */

    WaitUsec( 40 ) ;
}


void  lcd_putstr( char *buf, unsigned int line )
{
    unsigned int  data, rs ;
    char  *pnt ;

    if( line == 0 ) {
        data = 0x80 ;
    } else {
        data = 0xc0 ; /* 0x80 + 0x40(=adrs) */
    }
    rs = 0x000 ;
    lcd_putchar( data, rs ) ;

    rs = 0x100 ;
    pnt = buf ;
    while( (data=((unsigned int)*pnt++)&0xff) != 0x00 ) {
        lcd_putchar( data, rs ) ;
    }
}


void  lcd_clear( void )
{
    unsigned int  data, rs ;

    /* all clear */
    data = 0x01 ;
    rs = 0x000 ;
    lcd_putchar( data, rs ) ;
    WaitUsec( 1600 ) ;
}


/*
 * LCD initialize
 */
void  lcd_init( void )
{
    unsigned int  data, rs ;

    FM3_GPIO->ADE = FM3_GPIO->ADE & ~0x000000fc ;  /* ADC OFF:AN02-07 */

    /* set D4-D7 pin */
    FM3_GPIO->PFR1  = FM3_GPIO->PFR1  & ~0x00f0 ; /* set GPIO */
    FM3_GPIO->PCR1  = FM3_GPIO->PCR1  & ~0x00f0 ; /* off pull-up */
    FM3_GPIO->DDR1  = FM3_GPIO->DDR1  |  0x00f0 ; /* set output mode */
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x00f0 ; /* off */

    /* set RS,E pin */
    FM3_GPIO->PFR1  = FM3_GPIO->PFR1  & ~0x000c ; /* set GPIO */
    FM3_GPIO->PCR1  = FM3_GPIO->PCR1  & ~0x000c ; /* off pull-up */
    FM3_GPIO->DDR1  = FM3_GPIO->DDR1  |  0x000c ; /* set output mode */
    FM3_GPIO->PDOR1 = FM3_GPIO->PDOR1 & ~0x000c ; /* off */

    WaitMsec( 40 ) ;

    rs = 0x000 ;

    /* set 8bit to 4bit mode */
    data = 0x30 ;
    LCD_put4bit( data, rs ) ;
    WaitMsec( 4 ) ;
    WaitUsec( 100 ) ;

    data = 0x30 ;
    LCD_put4bit( data, rs ) ;
    WaitUsec( 100 ) ;

    data = 0x30 ;
    LCD_put4bit( data, rs ) ;

    data = 0x20 ;
    LCD_put4bit( data, rs ) ;
    WaitUsec( 1600 ) ;

    data = 0x28 ;
    lcd_putchar( data, rs ) ;

    data = 0x08 ;
    lcd_putchar( data, rs ) ;

    /* all clear */
    data = 0x01 ;
    lcd_putchar( data, rs ) ;
    WaitUsec( 1600 ) ;

    data = 0x06 ;
    lcd_putchar( data, rs ) ;

    data = 0x0c ;
    lcd_putchar( data, rs ) ;

    WaitMsec( 2 ) ;
}


//@}

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
}
#endif
/**INDENT-ON**/
/// @endcond
