/*
 *  for MB9AF312K on USB STICK
 * 
 *   Copyright (C) by Y.Ishioka.
 */
#include "../chip.h"
#include "sysdef_depend.h"
#include "string.h"

#include "fm3.h"

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
extern "C" {
#endif
/**INDENT-ON**/
/// @endcond


#define  CORE_CLOCK   40000000
#define  PERI2_CLOCK  (CORE_CLOCK/2)

#define  UART_DEF_BAUDRATE  115200

UW uart_port_base[] = {
    UART_S0_BASE,
    UART_S1_BASE,
    UART_S2_BASE,
    UART_S3_BASE,
    UART_S4_BASE
};

#if 0
/**********************************/
static  char  in_b( int port )
{
    return( 0 ) ;
}

static  short int  in_h( int port )
{
    return( 0 ) ;
}

static  int  in_w( int port )
{
    return( 0 ) ;
}

static  void  out_b( int port, char data )
{
}

static  void  out_h( int port, short int data )
{
}

static  void  out_w( int port, int data )
{
}

/**********************************/
#endif

/*
 * UART Send
 *   port==0 : port0
 *   port==1 : port1
 *   port==2 : port2
 *   port==3 : port3
 *   port==4 : port4
 */
void  uart_send( int port, unsigned char data )
{
    /* check busy */
    while( (in_b(uart_port_base[port]+UART_SSR)&UART_SSR_TDRE) == 0 ) ;

    out_b( uart_port_base[port]+UART_TDR, data ) ;
}


/*
 *  UART Receive
 *   port==0 : port0
 *   port==1 : port1
 *   port==2 : port2
 *   port==3 : port3
 *   port==4 : port4
 */
int  uart_recv( int port )
{
    unsigned char  data ;

    if( (in_b((UW)(uart_port_base[port]+UART_SSR))&UART_SSR_RDRF) == 0 ) { /* Rx empty */
        return( -1 ) ;
    }

    /* input data */
    data = in_b( (UW)(uart_port_base[port]+UART_RDR) ) ;

    return( (int)data ) ;
}


/*
 *   port==0 : port0
 *   port==1 : port1
 *   port==2 : port2
 *   port==3 : port3
 *   port==4 : port4
 */
void  uart_putstring( int port, unsigned char *buf )
{
    int  size = strlen( (const char *)buf ) ;
    int  loop ;

    for( loop=0 ; loop<size ; loop++ ) {
        uart_send( port, *buf++ ) ;
    }
}


/*
 *
 */
static void  uart_init_sub( UW base, UW baud )
{
    UW  size32 ;

    out_b( (UW)(base+UART_SMR), UART_SMR_MD_UART|UART_SMR_MD_SOE ) ;
    out_b( (UW)base+UART_SCR, 0x00 ) ;

    size32 = PERI2_CLOCK /  baud - 1 ;

    out_h( (UW)base+UART_BGR, (UH)size32 ) ;

    out_b( (UW)base+UART_SCR, (UART_SCR_TXE|UART_SCR_RXE) ) ;
}


/*
 *   port==0 : port0
 *   port==1 : port1
 *   port==2 : port2
 *   port==3 : port3
 *   port==4 : port4
 */
void  uart_chg_baud( int port, unsigned int baud )
{
    uart_init_sub( uart_port_base[port], (UW)baud ) ;
}


/*
 *   port==0 : port0
 *   port==1 : port1
 *   port==2 : port2
 *   port==3 : port3
 *   port==4 : port4
 */
void  uart_init( int port )
{
    UW  base ;

    if( port < 0 || 0 < port ) {
        return ;
    }

    base = uart_port_base[port] ;

  switch( port ) {
  case  0:
    out_w( GPIO_PFR2, in_w(GPIO_PFR2)|(1<<1) ) ;    // P21/SIN0_0
    out_w( GPIO_EPFR07, (in_w(GPIO_EPFR07)& ~(0x3<<4))|(0x1<<4) ) ;
    out_w( GPIO_PFR2, in_w(GPIO_PFR2)|(1<<2) ) ;    // P22/SOT0_0
    out_w( GPIO_EPFR07, (in_w(GPIO_EPFR07)& ~(0x3<<6))|(0x1<<6) ) ;
    out_w( GPIO_ADE, in_w(GPIO_ADE) & ~0x00000080 ) ;
    uart_init_sub( base, UART_DEF_BAUDRATE ) ;
    out_b( base+UART_ESCR, 0x00 ) ;
    break ;
  }

    out_b( base+UART_ESCR, in_b(base+UART_ESCR) | 0x80 ) ;
    out_b( base+UART_FCR0, 0x00 ) ;
    out_b( base+UART_FCR1, 0x00 ) ;
    out_b( base+UART_FBYTE1, 0x00 ) ;
    out_b( base+UART_FBYTE2, 0x00 ) ;
}

//@}

/// @cond 0
/**INDENT-OFF**/
#ifdef __cplusplus
}
#endif
/**INDENT-ON**/
/// @endcond
